#include "network_config.hh"

void Space (unsigned int depth)
{
  for (unsigned int i=0; i < depth; i++)
    std::cout << "\t";
}

void PrintGroup (kvtree::Group* g)
{
  using namespace kvtree::node_types;
  using namespace network_config;

  switch (g -> Type ()) {

  case interface_section:
    {
      InterfaceSection* is = (InterfaceSection *) g;
      std::cout << "interface " << is -> interface;
      if (!is -> Enabled())
	std::cout << " DISABLED";
      std::cout  << " " << is -> profile;
    }
    break;

  case address_subsection:
    {
      AddressSubsection* as = (AddressSubsection *) g;
      std::cout << "address part, dhcp=" << as -> Dhcp ();

      int count = 0;
      while (as->Address (count++));
      std::cout << " " << count - 1 << " ip addresses, ";

      count = 0;
      while (as->Gateway (count++));
      std::cout << count - 1 << " gateways, ";

      count = 0;
      while (as->Search (count++));
      std::cout << count - 1 << " search domains, ";

      count = 0;
      while (as->Nameserver (count++));
      std::cout << count - 1 << " nameservers.";

    }
    break;

  case wireless_subsection:
    {
      // TODO: test info functions
      std::cout << "wireless configuration";
    }
    break;

  case ppp_subsection:
    {
      PPPSubsection* ps = (PPPSubsection *)g;
      std::cout << "ppp setup with ";

      int count = 0;
      while (ps->Option (count++));
      std::cout << count - 1 << " ppp options"
		<< " pppoe=" << ps -> OverEthernet ()
		<< " defaults=" << ps -> Defaults ()
		<< " speed defaults=" << ps -> SpeedDefaults ()
		<< " chat defaults=" << ps -> ChatDefaults ();

    }
    break;
    
  default:
    std::cout << "???";
  }

  std::cout << std::endl;
}

void PrintIP (const network_config::IP& a)
{
  std::cout << (int) a.address[0] << "."
	    << (int) a.address[1] << "."
	    << (int) a.address[2] << "."
	    << (int) a.address[3];
}


void PrintLine (kvtree::Line* l)
{
  using namespace kvtree::node_types;
  using namespace network_config;

  switch (l -> Type ()) {

  case ip_setup:
    {
      IPLine& ipl = *((IPLine*) l);
      std::cout << "ip address set to ";
      PrintIP(ipl.address);
      std::cout << " netmask bits: " << ipl.address.netmask_bits; 
    }
    break;

  case gw_setup:
    {
      GatewayLine& gwl = *((GatewayLine*) l);
      std::cout << "gateway, nexthop ";
      PrintIP (gwl.nexthop);
    }
    break;

  case nameserver:
    {
      NameserverLine& nsl = *((NameserverLine*) l);
      std::cout << "nameserver ip ";
      PrintIP (nsl.nameserver);
    }
    break;

  case search:
    {
      SearchLine& sl = *((SearchLine*) l);
      std::cout << "search domain " << sl.domain;
    }
    break;

  case wireless_mode:
    {
      ModeLine& ml = *((ModeLine*) l);
      std::cout << "operation mode ";
      if (ml.mode == ModeLine::adhoc)
	std::cout << "Ad-Hoc";
      else if (ml.mode == ModeLine::managed)
	std::cout << "Managed";
      else
	std::cout << "???";
    }
    break;

  case wireless_channel:
    {
      ChannelLine& cl = *((ChannelLine*) l);
      std::cout << "channel " << cl.channel;
    }
    break;

  case wireless_essid:
    {
      EssidLine& el = *((EssidLine*) l);
      std::cout << "essid is " << el.essid;
    }
    break;

  case wireless_key:
    {
      KeyLine& kl = *((KeyLine*) l);
      std::cout << "encryption key " << kl.key;
    }
    break;

  case ppp_device:
    {
      DeviceLine& dl = *((DeviceLine*) l);
      std::cout << "device " << dl.device;
      if (dl.speed != 0)
	std::cout << " at speed " << dl.speed;
    }
    break;

  case ppp_chat_init:
    {
      ChatInitLine& cil = *((ChatInitLine*) l);
      std::cout << "chat init " << cil.init;
    }
    break;

  case ppp_chat_dial:
    {
      ChatDialLine& cdl = *((ChatDialLine*) l);
      std::cout << "dial " << cdl.dial;
    }
    break;

  case ppp_option:
    {
      OptionLine& ol = *((OptionLine*) l);
      std::cout << "option " << ol.option << " " << ol.value;
    }
    break;

    /* skeleton
       case :
       {
       Line& l = *((Line*) l);
       std::cout << "" << ;
       }
       break;
    */
    

  case unknown:
    std::cout << "unknown line : '" << l -> line << "'";
    break;

  default:
    std::cout << "???";
  }

  std::cout << std::endl;
}

void WalkTree (kvtree::Group* g, unsigned int depth)
{ 
  for (unsigned int i = 0; i < g -> childs.size (); i++) {
    kvtree::kvNode* current = g -> childs[i];
    Space (depth);
    if (current -> isTerminal()) {
      std::cout << "item type " << current -> Type () << " : ";
      kvtree::Line* line = (kvtree::Line *) current; 
      PrintLine (line);
    } else {
      std::cout << "subgroup type " << current -> Type () << " : ";
      kvtree::Group* group = (kvtree::Group *) current; 
      PrintGroup (group);
      WalkTree (group, depth+1);
    }
  }

}
