#include "address_box.hh"
#include <klocale.h>
#include <qlabel.h>
#include <qvbox.h>

AddressBox::AddressBox (QWidget* parent)
  : QGroupBox (parent)
{
  setColumnLayout (2, Qt::Horizontal);
  setTitle (i18n("manual setup"));

  config = 0;
  nml = 0;
 
  ip_label = new QLabel ( this );
  ip_label -> setText (i18n("IP address"));
  ip = new IpFrame( this );

  netmask_bits_label = new QLabel ( this );
  netmask_bits_label -> setText (i18n("netmask"));
  netmask_bits = new IpFrame( this );

  gw_check = new QCheckBox ( this );
  gw_check -> setText (i18n("gateway"));
  gw = new IpFrame( this );

  nameserver_check = new QCheckBox ( this );
  nameserver_check -> setText (i18n("nameserver"));

  nameserver = new IpFrame( this );

  QObject::connect (gw_check, SIGNAL (toggled (bool)), this, SLOT (  GatewayEnabled (bool)) );
  QObject::connect (nameserver_check, SIGNAL (toggled (bool)), this, SLOT (  NameserverEnabled (bool)) );
}

void AddressBox::NewConfig (network_config::AddressSubsection* subsection)
{
  config = subsection;
  bool create_all = false;

  network_config::IPLine* ipl = config -> Address (0);
  if (!ipl) {
    ipl = new network_config::IPLine ("");
    ipl -> address.SetTo (192,168,1,10);
    ipl -> address.netmask_bits = 24;
    ipl -> Set ();
    config -> childs.push_back (ipl);
    create_all = true;
  }
  ip -> SetLine (ipl);

  if (nml)
    delete nml;
  nml = new  network_config::NetmaskLine (ipl);
  netmask_bits -> SetLine (nml);

  network_config::GatewayLine* gwl = config -> Gateway (0);
  if (!gwl) {
    gwl = new network_config::GatewayLine ("");
    gwl -> nexthop.SetTo (192,168,1,1);
    gwl -> Set ();
    config -> childs.push_back (gwl);
    config -> setSkipGateways (!create_all);
  }
  gw -> SetLine (gwl);

  gw_check -> setChecked (! config -> skipGateways ());

  network_config::NameserverLine* nsl = config -> Nameserver (0);
  if (!nsl) {
    nsl = new network_config::NameserverLine ("");
    nsl -> nameserver.SetTo (192,168,1,1);
    nsl -> Set ();
    config -> childs.push_back (nsl);
    config -> setSkipNameservers (!create_all);
  }
  nameserver -> SetLine (nsl);

  nameserver_check -> setChecked (! config -> skipNameservers ());  
}

void AddressBox::GatewayEnabled (bool state)
{
  config -> setSkipGateways (!state);
  gw -> setEnabled (state);
}

void AddressBox::NameserverEnabled (bool state)
{
  config -> setSkipNameservers (!state);
  nameserver -> setEnabled (state);
}

