#include "ip_frame.hh"

IpBlock::IpBlock (IpFrame* parent, unsigned int block_number)
  : KRestrictedLine (parent, 0, "1234567890")
{
  m_frame = parent;
  myblock = block_number;
  block = 0;

  setMaxLength (3);
  setAlignment (Qt::AlignRight);

  setMaximumWidth (fontMetrics().maxWidth() * 3);

  QObject::connect( this, SIGNAL(textChanged(const QString&)), this, SLOT(changeAddress(const QString&)) );
}

void IpBlock::Update ()
{
  if (!block)
    return;
  QString number;
  number.setNum (*block);
  setText (number);
}

void IpBlock::NewIpLine ()
{
  block = &(m_frame -> m_line -> GetIP() -> address[myblock]);
  Update ();
}

void IpBlock::changeAddress (const QString& new_number)
{
  if (!block)
    return;

  int result = new_number.toInt ();

  if (result > 255) {
    *block = 255;
    Update ();
  } else if (result < 0) { // should never happen ...
    *block = 0; 
    Update ();
  } else
    *block = (network_config::IP::block) result;

  m_frame -> m_line -> Set ();
  //  std::cout << m_frame -> m_line -> line << std::endl;
}


IpFrame::IpFrame (QWidget* parent) : QHBox (parent)
{
  m_line = 0;
  for (unsigned int i=0; i < 4; i++) {
    block [i] = new IpBlock (this, i);

    if (i < 3) {
      dot [i] = new QLabel (this);
      dot [i] -> setText (" . ");
    }
  }
}

void IpFrame::SetLine (network_config::IPBaseLine* ipl)
{
  m_line = ipl;
  for (unsigned int i=0; i < 4; i++)
    block[i] -> NewIpLine ();
}

